#include "/prelude/core.glsl"

/* Light Index Deduplication */

#ifdef MC_GL_VENDOR_INTEL
	// work around compiler bug
	layout(local_size_x = gl_MaxComputeWorkGroupSize.x, local_size_y = 1, local_size_z = 1) in;
#else
	layout(local_size_x = min(gl_MaxComputeWorkGroupSize.x, INDEX_SIZE), local_size_y = 1, local_size_z = 1) in;
#endif

const ivec3 workGroups = ivec3(1, 1, 1);

uniform bool rebuildIndex;
uniform vec3 cameraPositionFract, invCameraPositionDeltaInt;
uniform mat4 gbufferModelViewInverse;

#include "/buf/index.glsl"

shared uint sh_culled_len;
shared uint[index.data.length()] sh_index_data;
shared uint16_t[index.data.length()] sh_index_color;

void main() {
	// maybe we would average all the light colors here for ambient light color

	immut uint16_t local_invocation_i = uint16_t(gl_LocalInvocationIndex);

	if (rebuildIndex) {
		if (local_invocation_i == uint16_t(0u)) sh_culled_len = 0u;

		// if (index.queue > index.data.length()) { index.len = uint16_t(0u); return; }

		immut uint16_t len = min(uint16_t(readFirstInvocationARB(index.queue)), uint16_t(index.data.length()));
		for (uint16_t i = local_invocation_i; i < len; i += uint16_t(gl_WorkGroupSize.x)) {
			sh_index_data[i] = index.data[i];
			sh_index_color[i] = index.color[i];
		}

		barrier();

		for (uint16_t i = local_invocation_i; i < len; i += uint16_t(gl_WorkGroupSize.x)) {
			immut uint data = sh_index_data[i];
			immut uint16_t color = sh_index_color[i];

			bool unique = true;

			// cull identical lights
			for (uint16_t j = uint16_t(0u); unique && j < i; ++j) if (sh_index_data[j] == data && sh_index_color[j] == color) unique = false;

			// cull different colored lights at the same pos, comparing the color bits to make it deterministic
			for (uint16_t j = uint16_t(0u); unique && j < len; ++j) if (sh_index_data[j] == data && sh_index_color[j] < color) unique = false;

			if (unique) {
				immut uint16_t i = uint16_t(atomicAdd(sh_culled_len, 1u));
				index.data[i] = data;
				index.color[i] = color;
			}
		}

		barrier();
		/*
		groupMemoryBarrier();

		immut uint16_t culled_len = uint16_t(readFirstInvocationARB(sh_culled_len));
		for (uint16_t i = local_invocation_i; i < culled_len; i += uint16_t(gl_WorkGroupSize.x)) {
			sh_index_data[i] = index.data[i];
			sh_index_color[i] = index.color[i];
		}

		barrier();

		immut f16vec3 index_offset = f16vec3(-255.5 - cameraPositionFract - gbufferModelViewInverse[3].xyz);

		for (uint16_t i = local_invocation_i; i < culled_len; i += uint16_t(gl_WorkGroupSize.x)) {
			uint16_t k = uint16_t(0u);

			immut uint data = sh_index_data[i];
			immut f16vec3 pe = f16vec3(
				data & 511u,
				bitfieldExtract(data, 9, 9),
				bitfieldExtract(data, 18, 9)
			) + index_offset;
			immut float16_t dist = dot(pe, pe);

			for (uint16_t j = uint16_t(0u); j < culled_len; ++j) {
				if (j != i) {
					immut uint other_data = sh_index_data[j];
					immut f16vec3 other_pe = f16vec3(
						other_data & 511u,
						bitfieldExtract(other_data, 9, 9),
						bitfieldExtract(other_data, 18, 9)
					) + index_offset;
					immut float16_t other_dist = dot(other_pe, other_pe);

					if (other_dist < dist) k++;
				}
			}

			index.data[k] = data;
			index.color[k] = sh_index_color[i]; // does this require 'coherent' SSBO?
		}
		*/

		if (local_invocation_i == uint16_t(0u)) {
			index.queue = 0u;
			index.offset = vec3(0.0);
			index.len = uint16_t(sh_culled_len);
		}
	} else if (local_invocation_i == uint16_t(0u)) index.offset += invCameraPositionDeltaInt;
}