#include "/prelude/core.glsl"

/* RENDERTARGETS: 1,2,3,4 */
layout(location = 0) out vec4 colortex1;
layout(location = 1) out vec4 colortex2;
layout(location = 2) out vec2 colortex3;
layout(location = 3) out vec3 colortex4;
layout(depth_greater) out float gl_FragDepth;

uniform float alphaTestRef;
uniform sampler2D gtexture;

#include "/lib/tbn/fsh.glsl"
#include "/lib/octa_normal.glsl"
#include "/lib/luminance.glsl"
#include "/lib/material/normal.glsl"
#include "/lib/material/specular.glsl"
#include "/lib/un11_11_10.glsl"
#include "/lib/srgb.glsl"

in VertexData {
	layout(location = 2, component = 0) vec2 texel;
	layout(location = 3, component = 0) flat uint tint;
	layout(location = 3, component = 1) flat uint emission;
	layout(location = 3, component = 2) flat uint light;
	layout(location = 4) vec3 s_screen;

	#if !(NORMALS == 1 && defined MC_NORMAL_MAP)
		layout(location = 3, component = 3) flat uint mid_coord;
		layout(location = 5, component = 0) flat uint face_tex_size;
	#endif
} v;

void main() {
	f16vec4 color = f16vec4(texelFetch(gtexture, ivec2(v.texel), 0));
	if (color.a < float16_t(alphaTestRef)) discard;

	colortex3 = unpackUnorm2x16(v.light);
	colortex4 = v.s_screen;

	immut f16vec3 tint = unpack_un11_11_10(v.tint);
	color.rgb *= tint;

	immut mat3 tbn = get_tbn();

	#if NORMALS == 1 && defined MC_NORMAL_MAP
		immut f16vec3 w_tex_normal = normalize(f16vec3(tbn * sample_normal(texelFetch(normals, ivec2(v.texel), 0).rg)));
	#elif NORMALS == 2
		immut f16vec3 w_tex_normal = f16vec3(tbn[2]);
	#else
		immut vec2 atlas = vec2(textureSize(gtexture, 0));
		immut f16vec3 w_tex_normal = normalize(f16vec3(tbn * gen_normal(gtexture, tint, v.texel / atlas, v.mid_coord, v.face_tex_size, atlas, luminance(color.rgb))));
	#endif

	colortex2 = vec4(
		octa_encode(w_tex_normal),
		octa_encode(f16vec3(tbn[2]))
	);

	immut float16_t roughness =
		#if SM && defined MC_SPECULAR_MAP
			map_roughness(float16_t(texelFetch(specular, ivec2(v.texel), 0).SM_CH));
		#else
			gen_roughness(luminance(color.rgb), float16_t(0.8));
		#endif

	color.rgb = linear(color.rgb);

	color.rgb *= mix(f16vec3(-1.0), f16vec3(1.0), bvec3(
		v.emission & 1u,
		bitfieldExtract(v.emission, 1, 1),
		bitfieldExtract(v.emission, 2, 1)
	));

	// Set negative sign bit for colortex1 hand flag
	colortex1 = vec4(color.rgb, float(roughness * float16_t(-1.0)));
}