#include "/prelude/core.glsl"

uniform float day;
uniform int handLightLevel;
uniform mat3 normalMatrix;
uniform mat4 gbufferModelViewInverse, modelViewMatrix, projectionMatrix, shadowModelView, textureMatrix;
uniform sampler2D gtexture;

#if HAND_LIGHT
	#include "/buf/hand_light.glsl"
	#include "/lib/srgb.glsl"
#endif

in vec2 mc_midTexCoord, vaUV0;
in vec3 vaPosition;
in vec4 vaColor;

#include "/lib/mmul.glsl"
#include "/lib/tbn/vsh.glsl"
#include "/lib/norm_uv2.glsl"
#include "/lib/un11_11_10.glsl"
#include "/lib/sm/distort.glsl"

out VertexData {
	layout(location = 2, component = 0) vec2 texel;
	layout(location = 3, component = 0) flat uint tint;
	layout(location = 3, component = 1) flat uint emission;
	layout(location = 3, component = 2) flat uint light;
	layout(location = 4) vec3 s_screen;

	#if !(NORMALS == 1 && defined MC_NORMAL_MAP)
		layout(location = 3, component = 3) flat uint mid_coord;
		layout(location = 5, component = 0) flat uint face_tex_size;
	#endif
} v;

void main() {
	immut vec3 view = rot_trans_mmul(modelViewMatrix, vaPosition);
	gl_Position = proj_mmul(projectionMatrix, view);
	v.tint = pack_un11_11_10(vaColor.rgb);

	f16vec2 light = norm_uv2();
	light.y = min(light.y + float16_t(AMBIENT), float16_t(1.0));
	v.light = packUnorm2x16(light);

	immut vec2 atlas = vec2(textureSize(gtexture, 0));
	immut vec2 coord = rot_trans_mmul(textureMatrix, vaUV0);
	v.texel = coord * atlas;

	#if !(NORMALS == 1 && defined MC_NORMAL_MAP)
		immut uvec2 texels = uvec2(fma(abs(coord - mc_midTexCoord), atlas * 2.0, vec2(0.5)));
		v.mid_coord = packUnorm2x16(mc_midTexCoord);
		v.face_tex_size = bitfieldInsert(texels.x, texels.y, 16, 16);
	#endif

	v.emission = uint(handLightLevel);

	#if HAND_LIGHT
		if (handLightLevel > 0) {
			immut uvec3 scaled_color = uvec3(fma(linear(vaColor.rgb * textureLod(gtexture, mix(coord, mc_midTexCoord, 0.5), 0.0).rgb), vec3(255.0), vec3(0.5)));

			atomicAdd(hand_light.data.r, scaled_color.r);
			atomicAdd(hand_light.data.g, scaled_color.g);
			atomicAdd(hand_light.data.b, scaled_color.b);
			atomicAdd(hand_light.data.a, 1u);
		}
	#endif

	init_tbn_w();

	immut vec3 s_ndc = shadow_proj_scale * (mat3(shadowModelView) * rot_trans_mmul(gbufferModelViewInverse, view));

	v.s_screen = fma(vec3(distort(s_ndc.xy), s_ndc.z), vec3(0.5), vec3(0.5));
}