#include "/prelude/core.glsl"

/* RENDERTARGETS: 1 */
layout(location = 0) out vec4 colortex1;
layout(depth_greater) out float gl_FragDepth;

uniform float alphaTestRef;
uniform vec3 sunDirectionPlr;
uniform sampler2D gtexture;

#include "/lib/srgb.glsl"

#ifndef NO_SKYLIGHT
	uniform mat4 gbufferModelViewInverse;

	#include "/lib/luminance.glsl"
	#include "/lib/skylight.glsl"
	#include "/lib/material/specular.glsl"
	#include "/lib/material/normal.glsl"
	#include "/lib/sm/shadows.glsl"
#endif

#include "/lib/tbn/fsh.glsl"

in VertexData {
	layout(location = 2, component = 0) vec2 texel;
	layout(location = 3, component = 0) vec3 tint;

	#ifndef NO_SKYLIGHT
		layout(location = 4, component = 0) vec3 view;
		layout(location = 5, component = 0) vec3 s_ndc;

		#if !(NORMALS == 1 && defined MC_NORMAL_MAP)
			layout(location = 6, component = 0) flat uint mid_coord;
			layout(location = 6, component = 1) flat uint face_tex_size;
		#endif
	#endif
} v;

void main() {
	f16vec4 color = f16vec4(texelFetch(gtexture, ivec2(v.texel), 0));
	if (color.a < float16_t(alphaTestRef)) discard;

	immut f16vec3 tint = f16vec3(v.tint);
	color.rgb *= tint;

	#ifndef NO_SKYLIGHT
		#if SM && defined MC_SPECULAR_MAP
			//immut float16_t roughness = map_roughness(texelFetch(specular, ivec2(v.texel), 0).SM_CH);
		#else
			//immut float16_t roughness = gen_roughness(luminance(color.rgb * srgb(v.tint)), 0.8);
		#endif

		immut mat3 tbn = get_tbn();

		#if NORMALS == 1 && defined MC_NORMAL_MAP
			immut vec3 v_tex_normal = normalize(tbn * sample_normal(texelFetch(normals, ivec2(v.texel), 0).rg));
		#elif NORMALS == 2
			immut vec3 v_tex_normal = tbn[2];
		#else
			immut vec2 atlas = vec2(textureSize(gtexture, 0));
			immut vec3 v_tex_normal = tbn * gen_normal(gtexture, tint, v.texel / atlas, v.mid_coord, v.face_tex_size, atlas, luminance(color.rgb));
		#endif
	#endif

	color.rgb = linear(color.rgb);

	#ifndef NO_SKYLIGHT
		color.rgb *= skylight();
		// todo!() proper shadow map stuff
	#endif

	colortex1 = color;
}