#include "/prelude/core.glsl"

uniform vec3 shadowLightDirection;
uniform mat3 normalMatrix;
uniform mat4 gbufferModelViewInverse, modelViewMatrix, projectionMatrix, shadowModelView, textureMatrix;
uniform sampler2D gtexture;

in vec2 vaUV0;
in vec3 vaPosition;
in vec4 vaColor;

#include "/lib/vertex_block_light.glsl"

#if !defined NO_SKYLIGHT && !(NORMALS == 1 && defined MC_NORMAL_MAP)
	in vec2 mc_midTexCoord;
#endif

#include "/lib/mmul.glsl"
#include "/lib/tbn/vsh.glsl"
#include "/lib/sm/distort.glsl"
#include "/lib/srgb.glsl"

out VertexData {
	layout(location = 2, component = 0) vec2 texel;
	layout(location = 3, component = 0) vec3 tint;

	#ifndef NO_SKYLIGHT
		layout(location = 4, component = 0) vec3 view;
		layout(location = 5, component = 0) vec3 s_ndc;

		#if !(NORMALS == 1 && defined MC_NORMAL_MAP)
			layout(location = 6, component = 0) flat uint mid_coord;
			layout(location = 6, component = 1) flat uint face_tex_size;
		#endif
	#endif
} v;

void main() {
	immut vec3 view = rot_trans_mmul(modelViewMatrix, vaPosition);
	gl_Position = proj_mmul(projectionMatrix, view);

	init_tbn_v();

	immut vec3 pe = mat3(gbufferModelViewInverse) * view;
	v.tint = vaColor.rgb * vec3(srgb(indexed_block_light(pe, f16vec3(mat3(gbufferModelViewInverse) * tbn_comp.normal))));

	immut vec2 atlas = vec2(textureSize(gtexture, 0));
	immut vec2 coord = rot_trans_mmul(textureMatrix, vaUV0);
	v.texel = coord * atlas;

	#ifndef NO_SKYLIGHT
		v.view = view.xyz;

		#if !(NORMALS == 1 && defined MC_NORMAL_MAP)
			immut uvec2 texels = uvec2(fma(abs(coord - mc_midTexCoord), atlas * 2.0, vec2(0.5)));
			v.mid_coord = packUnorm2x16(mc_midTexCoord);
			v.face_tex_size = bitfieldInsert(texels.x, texels.y, 16, 16);
		#endif

		immut float lambertian = dot(tbn_comp.normal, shadowLightDirection);

		vec3 s_ndc = shadow_proj_scale * (mat3(shadowModelView) * (pe + gbufferModelViewInverse[3].xyz));
		s_ndc.xy = distort(s_ndc.xy);
		float s_bias = 0.00005 / lambertian;

		#if SSS && defined TERRAIN
			// todo!() check that this makes sense
			if (mc_Entity.x == 0.0 && lambertian < 0.0) s_bias *= -1.0;
		#endif

		s_ndc.z -= s_bias;
		v.s_ndc = fma(s_ndc, vec3(0.5), vec3(0.5));
	#endif
}